@@ -46,6 +46,10 @@ async def index(request: web.Request) -> web.Response:
     # for example in the case of the WebTerminal.
     auth_only = request.query.get("auth_only")
 
+    host = request.headers.get("Host")
+    if "serve" in host:
+        redirect_to = "https://" + host
+
     path = {
         "tensorboard": "/tensorboard/",
         "grafana": "/grafana/",
@@ -55,29 +59,36 @@ async def index(request: web.Request) -> web.Response:
         "anyscaled": "/anyscaled/",
         "metrics": "/metrics",
         "autoscalermetrics": "/autoscalermetrics",
+        "serve": "/serve",
     }
 
-    if not token or not redirect_to or redirect_to not in path.keys():
+    if (
+        not token
+        or not redirect_to
+        or ("serve" not in host and redirect_to not in path)
+    ):
         return web.Response(
             text="token or redirect_to field not found, "
             "maybe you forgot to add `?token=..&redirect_to.` field? "
-            "redirect_to={tensorboard, dashboard, grafana, webterminal}.",
+            "redirect_to={tensorboard, dashboard, grafana, webterminal, serve}.",
             status=401,
         )
 
-    resp = (
-        web.HTTPFound(path[redirect_to])
-        if auth_only is None
-        else web.Response(text="ok")
-    )
+    if auth_only is None and "serve" in host:
+        resp = web.HTTPFound(redirect_to)
+    elif auth_only is None:
+        resp = web.HTTPFound(path[redirect_to])
+    else:
+        resp = web.Response(text="ok")
+
     use_secure = (
-        ANYSCALE_ENV["ANYSCALE_HOST"] == "https://beta.anyscale.com"
-        and redirect_to != "dashboard"
-    )
+        ANYSCALE_ENV.get("ANYSCALE_HOST") == "https://beta.anyscale.com"
+        or ANYSCALE_ENV.get("ANYSCALE_HOST") == "https://console.anyscale.com"
+    ) and redirect_to != "dashboard"
     if redirect_to == "webterminal":
         # The default for samesite is None currently but it is migrating
         # to Lax for most browser. Explicitly setting it here will
-        # enable beta.anyscale.com to communicate with webterminal service.
+        # enable console.anyscale.com to communicate with webterminal service.
         # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Set-Cookie/SameSite
         resp.set_cookie("anyscale-token", token, secure=True, samesite="None")
     else:
@@ -87,7 +98,7 @@ async def index(request: web.Request) -> web.Response:
 
 async def authorize(request: web.Request) -> web.Response:
     print(
-        "Got authorizatoin request for:",
+        "Got authorization request for:",
         request.headers.get("X-Forwarded-Uri", "unknown"),
     )
     cookies = request.cookies
@@ -105,7 +116,12 @@ async def authorize(request: web.Request) -> web.Response:
                 and token_list[0] == request.config_dict["auth_token"]
             ):
                 return web.Response(text="Authorized", status=200)
-        unauthorized_message = "Unauthorized. Please navigate to this resource through the cluster's page in the Anyscale product. Doing so will automatically authorize this request."
+        unauthorized_message = (
+            "Unauthorized. Please navigate to this resource through the cluster's page in the Anyscale "
+            "product. Doing so will automatically authorize this request from a web browser. Please "
+            "include the `anyscale-token` in the cookies if not performing this request from a web "
+            "browser."
+        )
         return web.Response(text=unauthorized_message, status=401)
 
 
@@ -124,7 +140,7 @@ def make_auth_proxy_app(auth_token: str) -> web.Application:
     # this server runs on the head node.
     # If its a dictionary, use the ANYSCALE_HOST as whitelisted host
     # these values could be an IP address in developer instances
-    # or beta.anyscale.com in production
+    # or console.anyscale.com in production
     if "ANYSCALE_HOST" in ANYSCALE_ENV:
         origins = [ANYSCALE_ENV["ANYSCALE_HOST"]]
 
