@@ -23,6 +23,8 @@ from ray.autoscaler.tags import (
 from urllib3.exceptions import HTTPError  # type: ignore
 import wrapt
 
+from anyscale.authenticate import get_auth_api_client
+
 
 # This try block exists because
 # 1. This module is passed to the autoscaler on the head node as an external node provider
@@ -37,7 +39,6 @@ except ModuleNotFoundError:
         NODE_PROVIDERS as _NODE_PROVIDERS,
     )
 
-from anyscale.api import instantiate_api_client
 from anyscale.client.openapi_client.api.default_api import DefaultApi
 from anyscale.client.openapi_client.models.add_instance_pool_member import (
     AddInstancePoolMember,
@@ -99,7 +100,7 @@ class AnyscaleInstanceManagerNodeProvider:
         cli_token = self.inner_provider_config["cli_token"]
         host = self.inner_provider_config["host"]
         self.api_client = wrap_api_client_with_retries(
-            instantiate_api_client(cli_token=cli_token, host=host)
+            get_auth_api_client(cli_token=cli_token, host=host).api_client
         )
         self.is_provider_supported = self._is_provider_supported()
 
@@ -214,6 +215,23 @@ class AnyscaleInstanceManagerNodeProvider:
         )
         return resp.result.internal_ip
 
+    def create_node_with_resources(
+        self,
+        node_config: Dict[str, Any],
+        tags: Dict[str, str],
+        count: int,
+        resources: Dict[str, float],
+    ) -> None:
+        """Mirrors Ray's NodeProvider interface. Needed for compatibility with
+        Ray Autoscaler version >=1.8.0
+
+        Refer to
+        https://github.com/anyscale/product/pull/8469
+        https://github.com/ray-project/ray/blob/releases/1.8.0/python/ray/autoscaler/node_provider.py#L127
+        https://github.com/ray-project/ray/blob/releases/1.8.0/python/ray/autoscaler/_private/node_launcher.py#L69
+        """
+        self.create_node(node_config, tags, count)
+
     def create_node(
         self, node_config: Dict[str, Any], tags: Dict[str, str], count: int
     ) -> None:
