@@ -9,11 +9,10 @@ import time
 from typing import cast, Dict, Iterator, List, Optional
 
 import click
-from openapi_client.rest import ApiException
 
 import anyscale
 from anyscale.client.openapi_client.api.default_api import DefaultApi
-from anyscale.util import format_api_exception, get_rsync_command, send_json_request
+from anyscale.util import get_rsync_command, send_json_request
 
 
 logger = logging.getLogger(__file__)
@@ -29,10 +28,9 @@ TIME_BETWEEN_HEARTBEATS = 30  # seconds
 @contextmanager
 def managed_autosync_session(session_id: str, api_client: DefaultApi) -> Iterator[str]:
     # Register to the API that we enabled autosync
-    with format_api_exception(ApiException):
-        resp = api_client.register_autosync_session_api_v2_autosync_sessions_post(
-            session_id
-        )
+    resp = api_client.register_autosync_session_api_v2_autosync_sessions_post(
+        session_id
+    )
 
     autosync_session_id = resp.result.id
     heartbeat_thread = AutosyncHeartbeat(autosync_session_id)
@@ -42,10 +40,9 @@ def managed_autosync_session(session_id: str, api_client: DefaultApi) -> Iterato
         yield autosync_session_id
     finally:
         heartbeat_thread.finish.set()
-        with format_api_exception(ApiException):
-            api_client.deregister_autosync_session_api_v2_autosync_sessions_autosync_session_id_delete(
-                autosync_session_id
-            )
+        api_client.deregister_autosync_session_api_v2_autosync_sessions_autosync_session_id_delete(
+            autosync_session_id
+        )
         heartbeat_thread.join()
         print("Autosync finished.")
 
