@@ -7,15 +7,15 @@ import os
 from typing import Optional
 
 import boto3
-from click import ClickException
+from click import ClickException, prompt
 
-from anyscale.api import get_api_client
-from anyscale.cli_logger import _CliLogger
-from anyscale.client.openapi_client.api.default_api import DefaultApi
+from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client.models.cloud_config import CloudConfig
 from anyscale.client.openapi_client.models.write_cloud import WriteCloud
 from anyscale.cloud import get_cloud_id_and_name, get_cloud_json_from_id
 import anyscale.conf
+from anyscale.controllers.base_controller import BaseController
+from anyscale.formatters import clouds_formatter
 from anyscale.util import (
     _CACHED_GCP_REGIONS,
     _get_role,
@@ -26,14 +26,13 @@ from anyscale.util import (
 )
 
 
-class CloudController(object):
+class CloudController(BaseController):
     def __init__(
-        self, api_client: DefaultApi = None, log: _CliLogger = _CliLogger(),
+        self, log: BlockLogger = BlockLogger(), initialize_auth_api_client: bool = True
     ):
-        if api_client is None:
-            api_client = get_api_client()
-        self.api_client = api_client
+        super().__init__(initialize_auth_api_client=initialize_auth_api_client)
         self.log = log
+        self.log.open_block("Output")
 
     def delete_cloud(
         self,
@@ -63,13 +62,39 @@ class CloudController(object):
 
         return True
 
+    def list_clouds(self, cloud_name: Optional[str], cloud_id: Optional[str]) -> str:
+        if cloud_id is not None:
+            clouds = [
+                self.api_client.get_cloud_api_v2_clouds_cloud_id_get(cloud_id).result
+            ]
+        elif cloud_name is not None:
+            clouds = [
+                self.api_client.find_cloud_by_name_api_v2_clouds_find_by_name_post(
+                    {"name": cloud_name}
+                ).result
+            ]
+        else:
+            clouds = self.api_client.list_clouds_api_v2_clouds_get().results
+        output = clouds_formatter.format_clouds_output(clouds=clouds, json_format=False)
+
+        return str(output)
+
     def setup_cloud(
-        self, provider: str, region: Optional[str], name: str, yes: bool = False
+        self,
+        is_aioa: bool,
+        provider: str,
+        region: Optional[str],
+        name: str,
+        yes: bool = False,
+        folder_id: Optional[int] = None,
     ) -> None:
         """
         Sets up a cloud provider
         """
 
+        if is_aioa:
+            return self.setup_aioa(provider=provider, region=region, name=name, yes=yes)
+
         if provider == "aws":
             # If the region is blank, change it to the default for AWS.
             if region is None:
@@ -82,20 +107,54 @@ class CloudController(object):
             # Warn the user about a bad region before the cloud configuration begins.
             # GCP's `list regions` API requires a project, meaning true verification
             # happens in the middle of the flow.
-            if region not in _CACHED_GCP_REGIONS:
+            if region not in _CACHED_GCP_REGIONS and not yes:
                 confirm(
                     f"You selected the region: {region}, but it is not in"
                     f"the cached list of GCP regions:\n\n{_CACHED_GCP_REGIONS}.\n"
                     "Continue cloud setup with this region?",
                     False,
                 )
-                # warn here
-            launch_gcp_cloud_setup(name=name, region=region)
+            if not yes and not folder_id:
+                folder_id = prompt(
+                    "Please select the GCP Folder ID where the 'Anyscale' folder will be created.\n"
+                    "\tYour GCP account must have permissions to create sub-folders in the specified folder.\n"
+                    "\tView your organization's folder layout here: https://console.cloud.google.com/cloud-resource-manager\n"
+                    "\tIf not specified, the 'Anyscale' folder will be created directly under the organization.\n"
+                    "Folder ID (numerals only)",
+                    default="",
+                    type=int,
+                    show_default=False,
+                )
+            # TODO: interactive setup process through the CLI?
+            launch_gcp_cloud_setup(name=name, region=region, folder_id=folder_id)
         else:
             raise ClickException(
                 f"Invalid Cloud provider: {provider}. Available providers are [aws, gcp]."
             )
 
+    def setup_aioa(
+        self, provider: str, region: Optional[str], name: str, yes: bool
+    ) -> None:
+        if provider == "aws":
+            if region is None:
+                region = "us-west-2"
+        else:
+            raise ClickException(
+                f"Cannot use {provider}. "
+                "Currently, only AWS is supported when running in Anyscale's cloud provider account."
+            )
+
+        self.api_client.create_cloud_api_v2_clouds_post(
+            write_cloud=WriteCloud(
+                provider=provider.upper(),
+                region=region,
+                credentials="",
+                name=name,
+                is_aioa=True,
+                is_k8s=True,  # All AIOA clouds are K8s clouds.
+            )
+        )
+
     def setup_aws(self, region: str, name: str, yes: bool = False) -> None:
         from ray.autoscaler._private.aws.config import DEFAULT_RAY_IAM_ROLE
 
@@ -112,9 +171,7 @@ class CloudController(object):
             yes,
         )
 
-        user_info = self.api_client.get_user_info_api_v2_userinfo_get().result
-
-        self.setup_aws_cross_account_role(region, user_info.id, name)
+        self.setup_aws_cross_account_role(region, name)
         self.setup_aws_ray_role(region, DEFAULT_RAY_IAM_ROLE)
 
         self.log.info("AWS credentials setup complete!")
@@ -125,9 +182,7 @@ class CloudController(object):
             "Head over to the web UI to create new sessions in your AWS account!"
         )
 
-    def setup_aws_cross_account_role(
-        self, region: str, user_id: str, name: str
-    ) -> None:
+    def setup_aws_cross_account_role(self, region: str, name: str) -> None:
         response = (
             self.api_client.get_anyscale_aws_account_api_v2_clouds_anyscale_aws_account_get()
         )
@@ -162,11 +217,7 @@ class CloudController(object):
 
         self.api_client.create_cloud_api_v2_clouds_post(
             write_cloud=WriteCloud(
-                provider="AWS",
-                region=region,
-                credentials=role.arn,
-                creator_id=user_id,
-                name=name,
+                provider="AWS", region=region, credentials=role.arn, name=name,
             )
         )
 
@@ -191,8 +242,7 @@ class CloudController(object):
 
             role = _get_role(role_name, region)
 
-        # Default permissions (copied from upstream)
-        role.attach_policy(PolicyArn="arn:aws:iam::aws:policy/AmazonEC2FullAccess")
+        # Modified permissions from Ray (no EC2FullAccess)
         role.attach_policy(PolicyArn="arn:aws:iam::aws:policy/AmazonS3FullAccess")
 
         # Also attach a role to allow it to launch more nodes with itself
