@@ -1,6 +1,7 @@
 from datetime import datetime
 import json
 import os
+from os import path
 import subprocess
 import sys
 from typing import Any, Dict, List, Optional, Tuple, Union
@@ -8,14 +9,13 @@ from typing import Any, Dict, List, Optional, Tuple, Union
 import click
 import yaml
 
-from anyscale.api import get_anyscale_api_client, get_api_client
 from anyscale.autosync_heartbeat import (
     get_working_dir_host_mount_location,
     managed_autosync_session,
     perform_autopush_synchronization,
     perform_sync,
 )
-from anyscale.cli_logger import _CliLogger
+from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client import (
     AppConfig,
     Build,
@@ -28,13 +28,13 @@ from anyscale.client.openapi_client import (
     SetupInitializeSessionOptions,
     StopSessionOptions,
 )
-from anyscale.client.openapi_client.api.default_api import DefaultApi
 from anyscale.cloud import get_cloud_id_and_name
 from anyscale.cluster_config import (
     _configure_min_max_workers,
     _update_file_mounts,
     get_cluster_config,
 )
+from anyscale.controllers.base_controller import BaseController
 from anyscale.project import (
     get_project_id,
     get_project_session,
@@ -74,19 +74,11 @@ def rsync(*args: Any, **kwargs: Any) -> None:
     ray_rsync(*args, **kwargs)
 
 
-class SessionController:
+class SessionController(BaseController):
     def __init__(
-        self,
-        api_client: Optional[DefaultApi] = None,
-        anyscale_api_client: Optional[DefaultApi] = None,
-        log: _CliLogger = _CliLogger(),
+        self, log: BlockLogger = BlockLogger(), initialize_auth_api_client: bool = True
     ):
-        if api_client is None:
-            api_client = get_api_client()
-        if anyscale_api_client is None:
-            anyscale_api_client = get_anyscale_api_client()
-        self.api_client = api_client
-        self.anyscale_api_client = anyscale_api_client
+        super().__init__(initialize_auth_api_client=initialize_auth_api_client)
         self.log = log
 
     def stop(
@@ -106,7 +98,7 @@ class SessionController:
         if not session_name and len(sessions) > 1:
             raise click.ClickException(
                 "Multiple active sessions: {}\n"
-                "Please specify the one you want to stop with --session-name.".format(
+                "Please specify the one you want to stop with --cluster-name.".format(
                     [session.name for session in sessions]
                 )
             )
@@ -147,7 +139,7 @@ class SessionController:
             )
         message = (
             "Warning: `anyscale start` has been deprecated. Please try "
-            "using `ray.client().connect()` for your use case and let the Anyscale team "
+            'using `ray.init("anyscale://cluster_name")` for your use case and let the Anyscale team '
             "know if you are having any issues."
         )
         self.log.warning(message)
@@ -178,7 +170,7 @@ class SessionController:
         )
 
         wait_for_session_start(project_id, session_name, self.api_client)
-        url = get_endpoint(f"/projects/{project_id}/sessions/{session.id}")
+        url = get_endpoint(f"/projects/{project_id}/clusters/{session.id}")
         print(f"Session {session_name} finished starting. View at {url}")
 
     def up(
@@ -200,10 +192,10 @@ class SessionController:
         dangerously_set_build_id: Optional[str] = None,
     ) -> None:
         message = (
-            "Warning: `anyscale up` is now deprecated in favor of ray.client().connect(). "
+            'Warning: `anyscale up` is now deprecated in favor of ray.init("anyscale://cluster_name"). '
             "`anyscale up` will still work for the time being, but we will remove it around "
             "the end of Q3. Please refer to https://docs.anyscale.com/api-sdk-reference/anyscale-ray-client "
-            "to learn more about using ray.client().connect() and let the Anyscale team know if you "
+            "to learn more about using ray.init() and let the Anyscale team know if you "
             "are running into any issues."
         )
         print("\033[91m{}\033[00m".format(message), file=sys.stderr)
@@ -367,7 +359,7 @@ class SessionController:
             )
 
             wait_for_session_start(project_id, session_name, self.api_client)  # type: ignore
-            url = get_endpoint(f"/projects/{project_id}/sessions/{session_id}")
+            url = get_endpoint(f"/projects/{project_id}/clusters/{session_id}")
             self.log.info(f"Session {session_name} started. View at {url}")
         except KeyboardInterrupt:
             self.api_client.stop_session_api_v2_sessions_session_id_stop_post(
@@ -407,40 +399,133 @@ class SessionController:
             cloned_session.project_id, cloned_session.name, self.api_client
         )
         url = get_endpoint(
-            f"/projects/{cloned_session.project_id}/sessions/{cloned_session.id}"
+            f"/projects/{cloned_session.project_id}/clusters/{cloned_session.id}"
         )
         return f"Session {cloned_session.name} started. View at {url}"
 
-    def ssh(self, session_name: str, ssh_option: Tuple[str]) -> None:
-        project_definition = load_project_or_throw()
-        project_id = get_project_id(project_definition.root)
-        session = get_project_session(project_id, session_name, self.api_client)
+    def ssh(
+        self,
+        session_name: str,
+        ssh_option: Tuple[str],
+        worker_node_id: Optional[str],
+        worker_node_ip: Optional[str],
+        force_head_node: bool,
+    ) -> None:
 
-        cluster_config = get_cluster_config(
-            session_name, self.api_client, disable_project_sync=True
-        )
+        config_path = "/home/ray/ray_bootstrap_config.yaml"
+        on_head_node = path.exists(config_path)
 
-        head_ip = self.api_client.get_session_head_ip_api_v2_sessions_session_id_head_ip_get(
-            session.id
-        ).result.head_ip
+        ssh_to_head = worker_node_id is None and worker_node_ip is None
 
-        ssh_user = cluster_config["auth"]["ssh_user"]
-        key_path = cluster_config["auth"]["ssh_private_key"]
-        container_name = get_container_name(cluster_config)
-
-        command = (
-            ["ssh"]
-            + list(ssh_option)
-            + ["-tt", "-i", key_path]
-            + ["{}@{}".format(ssh_user, head_ip)]
-            + (
-                [f"docker exec -it {container_name} sh -c 'which bash && bash || sh'"]
-                if container_name
-                else []
+        if force_head_node and ssh_to_head:
+            raise click.ClickException(
+                "Can't force head node if not sshing to a worker_node."
             )
-        )
 
-        subprocess.run(command)  # noqa: B1
+        if worker_node_id is not None and worker_node_ip is not None:
+            raise click.ClickException(
+                "A worker node id and worker node ip cannot both be set at once."
+            )
+
+        if force_head_node or (on_head_node and (worker_node_id or worker_node_ip)):
+            # On the head node
+            print("SSHing from head node to worker node.")
+
+            # Import ray where it's needed here to avoid making ray a
+            # regular dependency of the CLI. This should only execute on
+            # a head node where we can be confident ray will already
+            # be installed.
+            import ray
+
+            with open(config_path, "r") as f:
+                config_dict = yaml.safe_load(f)
+
+            try:
+                ssh_user = config_dict["auth"]["ssh_user"]
+                ssh_key_path = config_dict["auth"]["ssh_private_key"]
+                container_name = config_dict["docker"]["container_name"]
+            except ValueError as e:
+                raise click.ClickException(
+                    f"Could not parse ~/ray_bootstrap_config.yaml: {e}"
+                )
+
+            if worker_node_ip is not None:
+                node_ip = worker_node_ip
+            elif worker_node_id is not None:
+                ray.init(address="auto")
+                nodes = ray.nodes()
+                node_info = next(
+                    (node for node in nodes if node.get("NodeID") == worker_node_id),
+                    None,
+                )
+                if node_info is None:
+                    raise click.ClickException(
+                        f"Could not find a worker node with id {worker_node_id}."
+                    )
+                node_ip = node_info.get("NodeManagerAddress")
+
+            ssh_command = (
+                ["ssh"]
+                + list(ssh_option)
+                + ["-tt", "-i", ssh_key_path]
+                + ["{}@{}".format(ssh_user, node_ip)]
+                + [f"docker exec -it {container_name} sh -c 'which bash && bash || sh'"]
+            )
+        else:
+            # On a laptop
+
+            project_definition = load_project_or_throw()
+            project_id = get_project_id(project_definition.root)
+            session = get_project_session(project_id, session_name, self.api_client)
+
+            cluster_config = get_cluster_config(
+                session_name, self.api_client, disable_project_sync=True
+            )
+
+            head_ip = self.api_client.get_session_head_ip_api_v2_sessions_session_id_head_ip_get(
+                session.id
+            ).result.head_ip
+
+            ssh_user = cluster_config["auth"]["ssh_user"]
+            key_path = cluster_config["auth"]["ssh_private_key"]
+            container_name = get_container_name(cluster_config)
+
+            base_ssh_command = (
+                ["ssh"]
+                + list(ssh_option)
+                + ["-tt", "-i", key_path]
+                + ["{}@{}".format(ssh_user, head_ip)]
+            )
+
+            if ssh_to_head:
+                ssh_command = base_ssh_command + (
+                    [
+                        f"docker exec -it {container_name} sh -c 'which bash && bash || sh'"
+                    ]
+                    if container_name
+                    else []
+                )
+            else:
+                if not container_name:
+                    raise click.ClickException(
+                        "Cannot ssh to a worker node if this cluster has no container."
+                    )
+                # Don't use the ssh_options for the second ssh command because
+                # they probably refer to files that exist on the laptop but not
+                # the head node.
+
+                if worker_node_id is not None:
+                    ssh_flag = f"--worker-node-id {worker_node_id}"
+                else:
+                    ssh_flag = f"--worker-node-ip {worker_node_ip}"
+                ssh_from_head_command = (
+                    f"'anyscale ssh {ssh_flag} --force-head-node True'"
+                )
+                ssh_command = base_ssh_command + (
+                    [f"docker exec -it {container_name} sh -c  {ssh_from_head_command}"]
+                )
+
+        subprocess.run(ssh_command)  # noqa: B1
 
     def autopush(
         self,
@@ -451,11 +536,11 @@ class SessionController:
     ) -> None:
         message = (
             "Warning: `anyscale autopush` is now deprecated as we encourage users to start "
-            "clusters with ray.client().connect() and runtime environments. "
+            'clusters with ray.init("anyscale://cluster_name") and runtime environments. '
             "`anyscale autopush` will still work for the time being for clusters started "
             "with the sdk, but we will remove it around the end of Q3. Please refer to "
             "https://docs.anyscale.com/api-sdk-reference/anyscale-ray-client "
-            "to learn more about using ray.client().connect() and let the Anyscale team know if you "
+            "to learn more about using ray.init() and let the Anyscale team know if you "
             "are running into any issues."
         )
         print("\033[91m{}\033[00m".format(message), file=sys.stderr)
@@ -545,6 +630,8 @@ class SessionController:
         target: Optional[str] = None,
         config: Optional[str] = None,
     ) -> None:
+        message = "Warning: `anyscale pull` has been deprecated and will be removed in the future. "
+        self.log.warning(message)
         project_definition = load_project_or_throw()
 
         try:
@@ -611,13 +698,11 @@ class SessionController:
                 "added during autoscaling. Going forward, Anyscale will only support syncing "
                 "working directory files to the head and workers."
             )
-        if (source or target) and not no_warning:
+        if not no_warning:
             message = (
-                "Warning: The --source and --target options will soon be deprecated "
-                "for `anyscale push`. Going forward, Anyscale will only support syncing "
-                "the project directory files (which can be done without passing any "
-                "--source any --target arguments). Please let the Anyscale team know if "
-                "this will block your use case."
+                "Warning: `anyscale push` has been deprecated and will be removed in the future. "
+                "Please sync files to the cluster using the `working_dir` in Ray client. Please "
+                "contact the Anyscale team if you are having any issues with this for your use case."
             )
             self.log.warning(message)
 
@@ -664,7 +749,7 @@ class SessionController:
                 cloud_name=None,
                 idle_timeout=None,
             )
-        url = get_endpoint(f"/projects/{project_id}/sessions/{session.id}")
+        url = get_endpoint(f"/projects/{project_id}/clusters/{session.id}")
         self.log.info(f"Pushed to session {session_name}. View at {url}")
 
     # Helpers
