@@ -5,16 +5,25 @@ import sys
 from typing import Any
 
 import click
-from packaging import version
 
-from anyscale.api import instantiate_api_client
-from anyscale.cli_logger import _CliLogger
-from anyscale.client.openapi_client.rest import ApiException
+from anyscale.cli_logger import BlockLogger
 from anyscale.commands.anyscale_api.api_commands import anyscale_api
 from anyscale.commands.cloud_commands import cloud_cli
+from anyscale.commands.cluster_commands import cluster_cli
+from anyscale.commands.cluster_compute_commands import cluster_compute_cli
+from anyscale.commands.cluster_env_commands import cluster_env_cli
+from anyscale.commands.config_commands import config_cli
 from anyscale.commands.exec_commands import anyscale_exec
+from anyscale.commands.job_commands import job_cli
 from anyscale.commands.list_commands import list_cli
-from anyscale.commands.project_commands import anyscale_clone, anyscale_init
+from anyscale.commands.migrate_commands import migrate_cli
+from anyscale.commands.project_commands import (
+    anyscale_clone,
+    anyscale_init,
+    project_cli,
+)
+from anyscale.commands.run_commands import anyscale_kill_job, anyscale_run
+from anyscale.commands.service_commands import service_cli
 from anyscale.commands.session_commands import (
     anyscale_autopush,
     anyscale_autosync,
@@ -34,7 +43,7 @@ from anyscale.util import init_sentry
 logger = logging.getLogger(__file__)
 logging.getLogger("botocore").setLevel(logging.CRITICAL)
 
-log = _CliLogger()  # CLI Logger
+log = BlockLogger()  # CLI Logger
 
 if anyscale.conf.AWS_PROFILE is not None:
     logger.info("Using AWS profile %s", anyscale.conf.AWS_PROFILE)
@@ -74,29 +83,14 @@ class AliasedGroup(click.Group):
 )
 @click.pass_context
 def cli(ctx: Any, version_flag: bool, show_json: bool) -> None:
-    init_sentry(ctx)
-
     # Use anyscale vendored Ray for the CLI:
     sys.path.insert(0, anyscale.ANYSCALE_RAY_DIR)
-
-    try:
-        api_client = instantiate_api_client(no_cli_token=True)
-        resp = api_client.get_anyscale_version_api_v2_userinfo_anyscale_version_get()
-        curr_version = anyscale.__version__
-        latest_version = resp.result.version
-        if version.parse(curr_version) < version.parse(latest_version):
-            message = "Warning: Using version {0} of anyscale. Please update the package using pip install anyscale -U to get the latest version {1}".format(
-                curr_version, latest_version
-            )
-            log.warning(message)
-    except ApiException as e:
-        logger.warning(
-            "Error {} while trying to get latest anyscale version number: {}".format(
-                e.status, e.reason
-            )
-        )
-    except Exception as e:
-        logger.warning(e)
+    if (
+        (ctx.invoked_subcommand not in {"help", "version"})
+        and ("--help" not in sys.argv)
+        and ("-h" not in sys.argv)
+    ):
+        init_sentry(ctx.invoked_subcommand)
 
     if version_flag:
         ctx.invoke(version_cli, show_json=show_json)
@@ -123,8 +117,15 @@ def anyscale_help(ctx: Any) -> None:
 
 cli.add_command(session_cli)
 cli.add_command(cloud_cli)
+cli.add_command(config_cli)
+cli.add_command(migrate_cli)
+cli.add_command(project_cli)
 cli.add_command(version_cli)
 cli.add_command(list_cli)
+cli.add_command(cluster_env_cli)
+cli.add_command(job_cli)
+cli.add_command(service_cli)
+cli.add_command(cluster_cli)
 
 cli.add_command(anyscale_init)
 cli.add_command(anyscale_up)
@@ -136,9 +137,12 @@ cli.add_command(anyscale_clone)
 cli.add_command(anyscale_ssh)
 cli.add_command(anyscale_start)
 cli.add_command(anyscale_exec)
+cli.add_command(anyscale_run)
+cli.add_command(anyscale_kill_job)
 cli.add_command(anyscale_pull)
 cli.add_command(anyscale_push)
 cli.add_command(anyscale_help)
+cli.add_command(cluster_compute_cli)
 
 # Commands to interact with the Anyscale API
 cli.add_command(anyscale_api)
