@@ -1,27 +1,30 @@
+from contextlib import contextmanager
 import datetime
 import json
 import logging
 import os
+import random
+import string
 import subprocess
 import sys
 import threading
 import time
-from typing import Any, cast, Dict, Iterator, List, Optional, Tuple
+from typing import Any, Callable, cast, Dict, Generator, Iterator, List, Optional, Tuple
 from urllib.parse import quote as quote_sanitize, urlencode, urljoin
 import webbrowser
 
 import click
 import requests
 from requests import Response
+from urllib3.exceptions import MaxRetryError
 import yaml
 
-from anyscale.api import get_api_client
-from anyscale.cli_logger import _CliLogger
+from anyscale.authenticate import get_auth_api_client
+from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client.api.default_api import DefaultApi
 from anyscale.client.openapi_client.models.user_info import UserInfo
 from anyscale.client.openapi_client.rest import ApiException
 import anyscale.conf
-from anyscale.credentials import load_credentials
 import anyscale.shared_anyscale_utils.conf as shared_anyscale_conf
 from anyscale.shared_anyscale_utils.util import get_container_name
 
@@ -30,7 +33,7 @@ logger = logging.getLogger(__file__)
 
 BOTO_MAX_RETRIES = 5
 
-log = _CliLogger()  # Anyscale CLI Logger
+log = BlockLogger()  # Anyscale CLI Logger
 
 # Cached 07-21-2021
 _CACHED_GCP_REGIONS = json.loads(
@@ -42,6 +45,17 @@ def confirm(msg: str, yes: bool) -> Any:
     return None if yes else click.confirm(msg, abort=True)
 
 
+class AnyscaleEndpointFormatter:
+    def __init__(self, host: Optional[str] = None):
+        self.host = host or shared_anyscale_conf.ANYSCALE_HOST
+
+    def get_endpoint(self, endpoint: str) -> str:
+        return str(urljoin(self.host, endpoint))
+
+    def get_job_endpoint(self, job_id: str) -> str:
+        return self.get_endpoint(f"/jobs/{job_id}")
+
+
 def get_endpoint(endpoint: str, host: Optional[str] = None) -> str:
     return str(urljoin(host or shared_anyscale_conf.ANYSCALE_HOST, endpoint))
 
@@ -53,8 +67,7 @@ def send_json_request_raw(
     cli_token: Optional[str] = None,
     host: Optional[str] = None,
 ) -> Response:
-    if anyscale.conf.CLI_TOKEN is None and not cli_token:
-        anyscale.conf.CLI_TOKEN = load_credentials()
+    get_auth_api_client(cli_token=cli_token, host=host)
 
     url = get_endpoint(endpoint, host=host)
     cookies = {"cli_token": cli_token or anyscale.conf.CLI_TOKEN}
@@ -94,7 +107,7 @@ def send_json_request(
         if resp.status_code == 500:
             raise click.ClickException(
                 "There was an internal error in this command. "
-                "Please report this to the Anyscale team at beta@anyscale.com "
+                "Please report this to the Anyscale team at support@anyscale.zendesk.com "
                 "with the token '{}'.".format(resp.headers["x-trace-id"])
             )
 
@@ -199,12 +212,14 @@ def get_available_regions() -> List[str]:
     return [region["RegionName"] for region in client.describe_regions()["Regions"]]
 
 
-def launch_gcp_cloud_setup(name: str, region: str) -> None:
+def launch_gcp_cloud_setup(name: str, region: str, folder_id: Optional[int]) -> None:
     # TODO: enforce uniqueness for user's clouds
     quote_safe_name = quote_sanitize(name, safe="")
-    query_params = urlencode({"region": region})
+    query_params = {"region": region}
+    if folder_id:
+        query_params["folder_id"] = str(folder_id)
     # TODO: Replace this with a proper endpoint
-    endpoint = f"/api/v2/clouds/gcp/create/{quote_safe_name}?{query_params}"
+    endpoint = f"/api/v2/clouds/gcp/create/{quote_safe_name}?{urlencode(query_params)}"
     full_url = get_endpoint(endpoint)
     print(
         f"Launching GCP Oauth Flow:\n{full_url}\n(If this window does not auto-launch, use the link above)"
@@ -277,7 +292,7 @@ def get_active_sessions(
 
 def get_project_directory_name(project_id: str, api_client: DefaultApi = None) -> str:
     if api_client is None:
-        api_client = get_api_client()
+        api_client = get_auth_api_client().api_client
 
     # TODO (yiran): return error early if project doesn't exist.
     resp = api_client.get_project_api_v2_projects_project_id_get(project_id)
@@ -312,9 +327,9 @@ def get_wheel_url(
 
     if sys_platform == "darwin":
         if py_version == "38":
-            platform = "macosx_10_13_x86_64"
+            platform = "macosx_10_15_x86_64"
         else:
-            platform = "macosx_10_13_intel"
+            platform = "macosx_10_15_intel"
     elif sys_platform == "win32":
         platform = "win_amd64"
     else:
@@ -337,56 +352,109 @@ def get_wheel_url(
     )
 
 
+@contextmanager
+def updating_printer() -> Generator[Callable[[str], None], None, None]:
+    import shutil
+
+    cols, _ = shutil.get_terminal_size()
+
+    def print_status(status: str) -> None:
+        lines = status.splitlines()
+        first_line = lines[0]
+        truncated_first_line = (
+            first_line[0:cols]
+            if len(first_line) <= cols and len(lines) == 1
+            else (first_line[0 : cols - 3] + "...")
+        )
+        # Clear the line first
+        print("\r" + " " * cols, end="\r")
+        print(truncated_first_line, end="", flush=True)
+
+    try:
+        yield print_status
+    finally:
+        # Clear out the status and return to the beginning to reprint
+        print("\r" + " " * cols, end="\r", flush=True)
+
+
 def wait_for_session_start(
-    project_id: str, session_name: str, api_client: Optional[DefaultApi] = None
+    project_id: str,
+    session_name: str,
+    api_client: Optional[DefaultApi] = None,
+    log: BlockLogger = log,
+    block_label: Optional[str] = None,
 ) -> str:
-    log.info(f"Waiting for cluster {session_name} to start.")
+    if block_label:
+        log.info(
+            f"Waiting for cluster {BlockLogger.highlight(session_name)} to start. This may take a few minutes",
+            block_label=block_label,
+        )
+    else:
+        log.info(
+            f"Waiting for cluster {session_name} to start. This may take a few minutes"
+        )
+
     if api_client is None:
-        api_client = get_api_client()
+        api_client = get_auth_api_client().api_client
 
-    while True:
-        sessions = api_client.list_sessions_api_v2_sessions_get(
-            project_id=project_id, name=session_name, active_only=False
-        ).results
+    with updating_printer() as print_status:
+        while True:
+            sessions = api_client.list_sessions_api_v2_sessions_get(
+                project_id=project_id, name=session_name, active_only=False
+            ).results
 
-        if len(sessions) > 0:
-            session = sessions[0]
+            if len(sessions) > 0:
+                session = sessions[0]
 
-            if session.state == "Running" and session.pending_state is None:
-                return cast(str, session.id)
+                if session.state == "Running" and session.pending_state is None:
+                    return cast(str, session.id)
 
-            # Check for start up errors
-            if (
-                session.state_data
-                and session.state_data.startup
-                and session.state_data.startup.startup_error
-            ):
+                # Check for start up errors
+                if (
+                    session.state_data
+                    and session.state_data.startup
+                    and session.state_data.startup.startup_error
+                ):
+                    raise click.ClickException(
+                        f"Error while starting cluster {session_name}: {session.state_data.startup.startup_error}"
+                    )
+                elif (
+                    session.state
+                    and "Errored" in session.state
+                    and session.pending_state is None
+                ):
+                    raise click.ClickException(
+                        f"Error while starting cluster {session_name}: Cluster startup failed due to an error ({session.state})."
+                    )
+                elif (
+                    session.state
+                    and session.state in {"Terminated", "Stopped"}
+                    and session.pending_state is None
+                ):
+                    # Cluster is created in Terminated state; Check pending state to see if it is pending transition.
+                    raise click.ClickException(
+                        f"Error while starting cluster {session_name}: Cluster is still in stopped/terminated state."
+                    )
+                elif (
+                    session.state_data
+                    and session.state_data.startup
+                    and session.state_data.startup.startup_progress
+                ):
+                    # Print the latest status
+                    print_status(
+                        "Starting up " + session.state_data.startup.startup_progress
+                    )
+                elif (
+                    session.state != "StartingUp"
+                    and session.pending_state == "StartingUp"
+                ):
+                    print_status("Waiting for start up...")
+            else:
                 raise click.ClickException(
-                    f"Error while starting session {session_name}: {session.state_data.startup.startup_error}"
+                    f"Error while starting cluster {session_name}: Cluster doesn't exist."
                 )
-            elif (
-                session.state
-                and "Errored" in session.state
-                and session.pending_state is None
-            ):
-                raise click.ClickException(
-                    f"Error while starting session {session_name}: Session startup failed due to an error ({session.state})."
-                )
-            elif (
-                session.state
-                and session.state in {"Terminated", "Stopped"}
-                and session.pending_state is None
-            ):
-                # Session is created in Terminated state; Check pending state to see if it is pending transition.
-                raise click.ClickException(
-                    f"Error while starting session {session_name}: Session is still in stopped/terminated state."
-                )
-        else:
-            raise click.ClickException(
-                f"Error while starting session {session_name}: Session doesn't exist."
-            )
 
-        time.sleep(5)
+            time.sleep(2)
 
 
 def wait_for_head_node_start(
@@ -395,13 +463,13 @@ def wait_for_head_node_start(
     session_id: str,
     api_client: Optional[DefaultApi] = None,
 ) -> str:
-    url = get_endpoint(f"/projects/{project_id}/sessions/{session_id}")
+    url = get_endpoint(f"/projects/{project_id}/clusters/{session_id}")
     log.info(
-        f"Waiting for head node of session {session_name} to start. "
+        f"Waiting for head node of cluster {session_name} to start. "
         f"Please view the startup logs at {url}"
     )
     if api_client is None:
-        api_client = get_api_client()
+        api_client = get_auth_api_client().api_client
 
     retry_count = 0
     while True:
@@ -424,11 +492,11 @@ def wait_for_head_node_start(
                 and session.state_data.startup.startup_error
             ):
                 raise click.ClickException(
-                    f"Error while starting session {session_name}: {session.state_data.startup.startup_error}"
+                    f"Error while starting cluster {session_name}: {session.state_data.startup.startup_error}"
                 )
             elif session.state and "Errored" in session.state:
                 raise click.ClickException(
-                    f"Error while starting session {session_name}: Session startup failed due to an error ({session.state})."
+                    f"Error while starting cluster {session_name}: Cluster startup failed due to an error ({session.state})."
                 )
             elif (
                 retry_count >= 2
@@ -436,13 +504,13 @@ def wait_for_head_node_start(
                 and session.state in {"Terminated", "Stopped"}
                 and session.pending_state is None
             ):
-                # Session is created in Terminated state; Ignore the first few state checks.
+                # Cluster is created in Terminated state; Ignore the first few state checks.
                 raise click.ClickException(
-                    f"Error while starting session {session_name}: Session is still in stopped/terminated state."
+                    f"Error while starting cluster {session_name}: Cluster is still in stopped/terminated state."
                 )
         else:
             raise click.ClickException(
-                f"Error while starting session {session_name}: Session doesn't exist."
+                f"Error while starting cluster {session_name}: Cluster doesn't exist."
             )
 
         time.sleep(5)
@@ -615,40 +683,49 @@ def canonicalize_remote_location(
     return remote_location
 
 
-def init_sentry(ctx: click.core.Context) -> None:
+def init_sentry(subcommand_group: str) -> None:
     # Only report production and staging errors
     if shared_anyscale_conf.ANYSCALE_ENV not in ["production", "staging"]:
         return
 
-    import sentry_sdk
+    try:
+        import sentry_sdk
 
-    user_info = get_user_info()
-    if user_info:
-        if user_info.email in [
-            "test@anyscale.com",
-            "ci-staging@anyscale.com",
-            "ci-production@anyscale.com",
-        ]:
-            return
-        sentry_sdk.set_user(
-            {
-                "id": user_info.id,
-                "email": user_info.email,
-                "username": user_info.username,
-            }
+        user_info = get_user_info()
+        if user_info:
+            if user_info.email in [
+                "test@anyscale.com",
+                "ci-staging@anyscale.com",
+                "ci-production@anyscale.com",
+            ]:
+                return
+            sentry_sdk.set_user(
+                {
+                    "id": user_info.id,
+                    "email": user_info.email,
+                    "username": user_info.username,
+                }
+            )
+
+        sentry_sdk.set_tag("command", subcommand_group)
+        sentry_sdk.init(
+            "https://68d7689b55024e91bb31694539111baa@o565360.ingest.sentry.io/5706818",
+            release=get_anyscale_version(),
+            environment=shared_anyscale_conf.ANYSCALE_ENV,
         )
-
-    sentry_sdk.set_tag("command", ctx.invoked_subcommand)
-    sentry_sdk.init(
-        "https://68d7689b55024e91bb31694539111baa@o565360.ingest.sentry.io/5706818",
-        release=get_anyscale_version(),
-        environment=shared_anyscale_conf.ANYSCALE_ENV,
-    )
+    except MaxRetryError as e:
+        log.debug(str(e))
+        log.warning(
+            "Warning: Anyscale requires a network connection to operate, are you currently "
+            "connected to the internet?"
+        )
+    except Exception as e:
+        log.warning(str(e))
 
 
 def get_user_info() -> Optional[UserInfo]:
     try:
-        api_client = get_api_client()
+        api_client = get_auth_api_client().api_client
     except click.exceptions.ClickException:
         return None
     return api_client.get_user_info_api_v2_userinfo_get().result
@@ -683,3 +760,18 @@ def get_anyscale_version() -> str:
         # None of this is critical behavior and so this should not crash the CLI if we can't get a git version.
         pass
     return anyscale.__version__
+
+
+def generate_slug(length: int = 6) -> str:
+    return "".join(random.choices(string.ascii_lowercase + string.digits, k=length))
+
+
+def validate_non_negative_arg(ctx, param, value):
+    """
+    Checks that an integer option to click command is non-negative.
+    """
+    if value < 0:
+        raise click.ClickException(
+            f"Please specify a non-negative value for {param.opts[0]}"
+        )
+    return value
