@@ -67,17 +67,15 @@ class PackagePrefix(Enum):
 
 
 def _gen_token() -> Any:
-    import anyscale
-    from anyscale.api import get_api_client
-    from anyscale.credentials import load_credentials
+    from anyscale.authenticate import get_auth_api_client
     from anyscale.sdk.anyscale_client.sdk import AnyscaleSDK
 
+    auth_api_client = get_auth_api_client()
     sdk = AnyscaleSDK(
-        auth_token=load_credentials(),
-        host=f"{anyscale.shared_anyscale_utils.conf.ANYSCALE_HOST}/ext",
+        auth_token=auth_api_client.credentials, host=auth_api_client.host,
     )
     configure_open_api_client_headers(sdk.api_client, "connect")
-    api_client = get_api_client()
+    api_client = auth_api_client.api_client
     configure_open_api_client_headers(api_client.api_client, "connect")
     org_id = api_client.get_user_info_api_v2_userinfo_get().result.organization_ids[0]
     token = sdk.get_organization_temporary_object_storage_credentials(
@@ -733,6 +731,9 @@ def rewrite_runtime_env_uris(job_config: "JobConfig") -> None:
         working_path, excludes, meta
     )
     # Put all uris into `uris` field.
+    # TODO(architkulkarni): Once runtime env URIs are downloaded in the
+    # agent, we will need to modify the below to use the new
+    # job_config.set_runtime_env_uris() API introduced in Ray 1.5.0
     job_config.runtime_env["uris"] = [p.uri() for p in file_pkgs]
     job_config.runtime_env["uris"].extend(
         [p.uri() for p in [base_pkg, add_pkg, del_pkg] if p is not None]
@@ -901,18 +902,6 @@ async def _prepare_conda_env(
             await _handle_cache_hit(s3, token, base_pkg, resource_dir_pkg, conda_env)
 
 
-# To make it compatible with shim process in oss ray
-parser = argparse.ArgumentParser()
-
-parser.add_argument(
-    "--serialized-runtime-env", type=str, help="the serialized parsed runtime env dict"
-)
-
-parser.add_argument(
-    "--session-dir", type=str, help="the directory for the current session"
-)
-
-
 def _get_conda_cache_uris(conda_env: Dict[str, Any]) -> Tuple[_PkgURI, _PkgURI]:
     """
     This function will rewrite conda env into uris
@@ -959,12 +948,13 @@ def _get_conda_dict(
     ray_pip = current_ray_pip_specifier()
     extra_pip_dependencies = []
     if "ANYSCALE_USER_WHEEL" in os.environ:
+        logger.debug(f"Using anyscale wheel {os.environ['ANYSCALE_USER_WHEEL']}")
         extra_pip_dependencies.append(os.environ["ANYSCALE_USER_WHEEL"])
     else:
         import anyscale
 
         extra_pip_dependencies.append(f"anyscale=={anyscale.__version__}")
-    if ray_pip:
+    if ray_pip and not runtime_env.get("_skip_inject_ray"):
         extra_pip_dependencies.append(ray_pip)
         extra_pip_dependencies.append("ray[default]")
     return inject_dependencies(conda_env, py_version, extra_pip_dependencies)
@@ -1003,9 +993,19 @@ def _get_conda_env_path(runtime_env: Dict[str, Any]) -> Optional[str]:
         return str(resource_dir_pkg.local_path().resolve())
 
 
+def _update_env_vars(runtime_env: Dict[str, Any]) -> None:
+    """This function will update the environment variables of the current
+    process based on the env_vars field of the runtime_env argument.
+    """
+    if runtime_env.get("env_vars"):
+        env_vars = runtime_env["env_vars"]
+        os.environ.update(env_vars)
+
+
 def worker_setup_func(input_args: Any) -> None:
     """This func will install conda env if not exists. And it'll activate it.
 
+    This function is only used for ray<1.5.
     conda env should be:
       {
           uris: [] // s3 pkgs. It can be base_/add_
@@ -1016,6 +1016,18 @@ def worker_setup_func(input_args: Any) -> None:
     # remaining_args contains the arguments to the original worker command,
     # minus the python executable, e.g. default_worker.py --node-ip-address=...
     logger.debug("Start to run shim process")
+    # To make it compatible with shim process in oss ray
+    parser = argparse.ArgumentParser()
+
+    parser.add_argument(
+        "--serialized-runtime-env",
+        type=str,
+        help="the serialized parsed runtime env dict",
+    )
+
+    parser.add_argument(
+        "--session-dir", type=str, help="the directory for the current session"
+    )
     args, remaining_args = parser.parse_known_args(args=input_args)
 
     commands = []
@@ -1025,13 +1037,32 @@ def worker_setup_func(input_args: Any) -> None:
     if conda_path is not None:
         commands += ray_conda.get_conda_activate_commands(conda_path)  # type: ignore
 
-    commands += [" ".join(["exec python"] + remaining_args)]
+    # After activating the conda environment, "python" will point to the
+    # correct python executable for that environment.  If no conda environment
+    # is specified, we should just use the current python executable.
+    py_executable: str = "python" if conda_path is not None else sys.executable
+
+    commands += [" ".join([f"exec {py_executable}"] + remaining_args)]
     command_separator = " && "
     command_str = command_separator.join(commands)
     logger.debug("Shim process setup finished.")
+
+    _update_env_vars(runtime_env)
+
     os.execvp("bash", ["bash", "-c", command_str])
 
 
+def setup_runtime_env(runtime_env: dict, session_dir):
+    """Used for Ray 1.5+.  Sets up the given runtime environment."""
+
+    from ray.workers.pluggable_runtime_env import RuntimeEnvContext
+
+    conda_env_name = _get_conda_env_path(runtime_env)
+    if conda_env_name is not None:
+        return RuntimeEnvContext(conda_env_name)
+    return RuntimeEnvContext()
+
+
 """
 The following functions are anyscale related functions
 """
@@ -1049,6 +1080,7 @@ def register_runtime_env(env: Dict[str, Any]) -> List[str]:
 
 
 def runtime_env_setup() -> None:
+    import ray
     import ray._private.runtime_env as ray_runtime_env
 
     ray_runtime_env.rewrite_runtime_env_uris = rewrite_runtime_env_uris
@@ -1059,6 +1091,22 @@ def runtime_env_setup() -> None:
 
     import ray.ray_constants as ray_constants
 
-    ray_constants.DEFAULT_WORKER_SETUP_HOOK = (
-        "anyscale.utils.runtime_env.worker_setup_func"
-    )
+    # TODO(architkulkarni): Refactor runtime_env so we have a stable interface
+    # between OSS and product.
+    if ray.__version__[:3] == "1.4":
+        ray_constants.DEFAULT_WORKER_SETUP_HOOK = (
+            "anyscale.utils.runtime_env.worker_setup_func"
+        )
+    elif ray.__version__[:3] in ["1.5", "1.6", "2.0"]:
+        import ray.workers.setup_runtime_env as ray_setup_runtime_env
+
+        ray_setup_runtime_env.setup_runtime_env = setup_runtime_env
+        ray_constants.DEFAULT_RUNTIME_ENV_SETUP_HOOK = (
+            "anyscale.utils.runtime_env.setup_runtime_env"
+        )
+    else:
+        raise ValueError(
+            "S3 caching for runtime envs is only supported on "
+            "Ray 1.5 and 1.6.  Current Ray version: "
+            f"{ray.__version__}."
+        )
