@@ -16,7 +16,6 @@ from anyscale.client.openapi_client import (
     ClusterConfig,
     ComputeTemplateConfig,
     CreateComputeTemplate,
-    CreateSessionFromSnapshotOptions,
     Project,
     ProjectDefaultSessionName,
     ProjectdefaultsessionnameResponse,
@@ -72,6 +71,48 @@ COMPUTE_CONFIG_TEMPLATE = json.dumps(
 )
 
 
+@pytest.fixture()
+def mock_auth_api_client(mock_api_client: Mock, base_mock_anyscale_api_client: Mock):
+    mock_auth_api_client = Mock(
+        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client,
+    )
+    with patch.multiple(
+        "anyscale.controllers.base_controller",
+        get_auth_api_client=Mock(return_value=mock_auth_api_client),
+    ):
+        yield
+
+
+@pytest.fixture()
+def mock_auth_api_client_multiple_sessions(
+    mock_api_client_multiple_sessions: Mock, base_mock_anyscale_api_client: Mock
+):
+    mock_auth_api_client = Mock(
+        api_client=mock_api_client_multiple_sessions,
+        anyscale_api_client=base_mock_anyscale_api_client,
+    )
+    with patch.multiple(
+        "anyscale.controllers.base_controller",
+        get_auth_api_client=Mock(return_value=mock_auth_api_client),
+    ):
+        yield
+
+
+@pytest.fixture()
+def mock_auth_api_client_with_session(
+    mock_api_client_with_session: Mock, base_mock_anyscale_api_client: Mock
+):
+    mock_auth_api_client = Mock(
+        api_client=mock_api_client_with_session,
+        anyscale_api_client=base_mock_anyscale_api_client,
+    )
+    with patch.multiple(
+        "anyscale.controllers.base_controller",
+        get_auth_api_client=Mock(return_value=mock_auth_api_client),
+    ):
+        yield
+
+
 @pytest.fixture()
 def mock_api_client(mock_api_client_with_session: Mock) -> Mock:
     mock_api_client = mock_api_client_with_session
@@ -82,14 +123,8 @@ def mock_api_client(mock_api_client_with_session: Mock) -> Mock:
     return mock_api_client
 
 
-def test_stop_single_session(
-    session_test_data: Session,
-    mock_api_client: Mock,
-    base_mock_anyscale_api_client: Mock,
-) -> None:
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
+def test_stop_single_session(session_test_data: Session, mock_auth_api_client) -> None:
+    session_controller = SessionController()
 
     mock_project_definition = Mock()
     mock_project_definition.root = "/some/directory"
@@ -106,7 +141,7 @@ def test_stop_single_session(
             "test-name", delete=False, workers_only=False, keep_min_workers=False,
         )
 
-    mock_api_client.stop_session_api_v2_sessions_session_id_stop_post.assert_called_once_with(
+    session_controller.api_client.stop_session_api_v2_sessions_session_id_stop_post.assert_called_once_with(
         session_test_data.id,
         StopSessionOptions(
             terminate=True, delete=False, workers_only=False, keep_min_workers=False
@@ -157,13 +192,8 @@ def mock_api_client_multiple_sessions(base_mock_api_client: Mock) -> Mock:
     return base_mock_api_client
 
 
-def test_stop_multiple_sessions(
-    mock_api_client_multiple_sessions: Mock, base_mock_anyscale_api_client: Mock
-) -> None:
-    mock_api_client = mock_api_client_multiple_sessions
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
+def test_stop_multiple_sessions(mock_auth_api_client_multiple_sessions) -> None:
+    session_controller = SessionController()
 
     mock_project_definition = Mock()
     mock_project_definition.root = "/some/directory"
@@ -180,7 +210,7 @@ def test_stop_multiple_sessions(
             "ses_?", delete=True, workers_only=True, keep_min_workers=True,
         )
 
-    mock_api_client.stop_session_api_v2_sessions_session_id_stop_post.assert_has_calls(
+    session_controller.api_client.stop_session_api_v2_sessions_session_id_stop_post.assert_has_calls(
         [
             call(
                 "ses_1",
@@ -239,13 +269,10 @@ def mock_up_tuple(
 
 
 def test_anyscale_up_success(
-    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock],
-    base_mock_anyscale_api_client: Mock,
+    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock], mock_auth_api_client
 ) -> None:
     (mock_api_client, project_test_data, cloud_test_data, _,) = mock_up_tuple
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
+    session_controller = SessionController()
     session_controller.push = Mock()  # type: ignore
     mock_api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get.return_value = ProjectdefaultsessionnameResponse(
         result=ProjectDefaultSessionName(name="session-11")
@@ -307,13 +334,10 @@ def test_anyscale_up_success(
 
 
 def test_anyscale_up_failure(
-    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock],
-    base_mock_anyscale_api_client: Mock,
+    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock], mock_auth_api_client
 ) -> None:
     (mock_api_client, _, _, mock_wait_for_start,) = mock_up_tuple
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
+    session_controller = SessionController()
     session_controller.push = Mock()  # type: ignore
     sesion_id = "session-10"
     mock_api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get.return_value = ProjectdefaultsessionnameResponse(
@@ -353,13 +377,10 @@ def test_anyscale_up_failure(
 
 
 def test_anyscale_up_no_config_provided(
-    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock],
-    base_mock_anyscale_api_client: Mock,
+    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock], mock_auth_api_client
 ) -> None:
     (mock_api_client, project_test_data, cloud_test_data, _,) = mock_up_tuple
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
+    session_controller = SessionController()
     session_controller.push = Mock()  # type: ignore
     mock_api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get.return_value = ProjectdefaultsessionnameResponse(
         result=ProjectDefaultSessionName(name="session-11")
@@ -424,13 +445,10 @@ def test_anyscale_up_no_config_provided(
 
 
 def test_anyscale_app_template(
-    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock],
-    base_mock_anyscale_api_client: Mock,
+    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock], mock_auth_api_client
 ) -> None:
     (mock_api_client, project_test_data, cloud_test_data, _,) = mock_up_tuple
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
+    session_controller = SessionController()
     session_controller.push = Mock()  # type: ignore
     mock_api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get.return_value = ProjectdefaultsessionnameResponse(
         result=ProjectDefaultSessionName(name="session-11")
@@ -493,17 +511,14 @@ def test_anyscale_app_template(
 
 
 def test_get_or_generate_session_name(
-    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock],
-    base_mock_anyscale_api_client: Mock,
+    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock], mock_auth_api_client
 ) -> None:
     (mock_api_client, project_test_data, _, _) = mock_up_tuple
     mock_api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get.return_value = ProjectdefaultsessionnameResponse(
         result=ProjectDefaultSessionName(name="session-11")
     )
 
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
+    session_controller = SessionController()
     session_controller._get_or_generate_session_name(None, project_test_data.id)
     mock_api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get.assert_called_once_with(
         project_id=project_test_data.id
@@ -511,16 +526,13 @@ def test_get_or_generate_session_name(
 
 
 def test_register_compute_config(
-    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock],
-    base_mock_anyscale_api_client: Mock,
+    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock], mock_auth_api_client
 ) -> None:
     (mock_api_client, project_test_data, _, _) = mock_up_tuple
     mock_api_client.create_compute_template_api_v2_compute_templates_post = Mock(
         return_value=Mock(result=Mock(id="mock_compute_template_id"))
     )
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
+    session_controller = SessionController()
     with tempfile.NamedTemporaryFile(mode="w") as temp_file:
         temp_file.write(COMPUTE_CONFIG_TEMPLATE)
         temp_file.flush()
@@ -540,21 +552,20 @@ def test_register_compute_config(
 @pytest.mark.parametrize("update_session", [False, True])
 def test_create_or_update_session_data(
     mock_up_tuple: Tuple[Mock, Project, Cloud, Mock],
-    base_mock_anyscale_api_client: Mock,
+    mock_auth_api_client,
     update_session: bool,
 ) -> None:
     (mock_api_client, project_test_data, _, _) = mock_up_tuple
+    session_controller = SessionController()
+    session_controller.api_client = mock_api_client
     if update_session:
-        mock_api_client.list_sessions_api_v2_sessions_get.return_value = SessionListResponse(
+        session_controller.api_client.list_sessions_api_v2_sessions_get.return_value = SessionListResponse(
             results=[Mock(id="session-11-id")]  # Session already exists.
         )
     else:
-        mock_api_client.list_sessions_api_v2_sessions_get.return_value = SessionListResponse(
+        session_controller.api_client.list_sessions_api_v2_sessions_get.return_value = SessionListResponse(
             results=[]  # No session with this name yet, brand new session.
         )
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
     session_controller._create_or_update_session_data(
         session_name="session-11",
         project_id=project_test_data.id,
@@ -564,7 +575,7 @@ def test_create_or_update_session_data(
     )
 
     if update_session:
-        base_mock_anyscale_api_client.update_session.assert_called_once_with(
+        session_controller.anyscale_api_client.update_session.assert_called_once_with(
             "session-11-id", UpdateSession(idle_timeout=-1,),
         )
     else:
@@ -578,53 +589,47 @@ def test_create_or_update_session_data(
             idle_timeout=-1,
             uses_app_config=True,
         )
-        base_mock_anyscale_api_client.create_session.assert_called_once_with(
+        session_controller.anyscale_api_client.create_session.assert_called_once_with(
             create_session_data
         )
 
 
 def test_list_app_configs(
-    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock],
-    base_mock_anyscale_api_client: Mock,
+    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock], mock_auth_api_client
 ) -> None:
     (mock_api_client, project_test_data, _, _) = mock_up_tuple
-    base_mock_anyscale_api_client.list_app_configs = Mock(
+    session_controller = SessionController()
+    session_controller.api_client = mock_api_client
+    session_controller.anyscale_api_client.list_app_configs = Mock(
         return_value=Mock(results=[], metadata=Mock(next_paging_token=None))
     )
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
     session_controller._list_app_configs(project_id=project_test_data.id)
-    base_mock_anyscale_api_client.list_app_configs.assert_called_once_with(
+    session_controller.anyscale_api_client.list_app_configs.assert_called_once_with(
         project_id=project_test_data.id, count=50, paging_token=None
     )
 
 
 def test_list_builds(
-    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock],
-    base_mock_anyscale_api_client: Mock,
+    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock], mock_auth_api_client
 ) -> None:
     (mock_api_client, _, _, _) = mock_up_tuple
-    base_mock_anyscale_api_client.list_builds = Mock(
+    session_controller = SessionController()
+    session_controller.api_client = mock_api_client
+    session_controller.anyscale_api_client.list_builds = Mock(
         return_value=Mock(results=[], metadata=Mock(next_paging_token=None))
     )
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
     session_controller._list_builds("mock_app_config_id")
-    base_mock_anyscale_api_client.list_builds.assert_called_once_with(
+    session_controller.anyscale_api_client.list_builds.assert_called_once_with(
         "mock_app_config_id", count=50, paging_token=None
     )
 
 
 def test_get_app_config_id_from_name(
-    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock],
-    base_mock_anyscale_api_client: Mock,
+    mock_up_tuple: Tuple[Mock, Project, Cloud, Mock], mock_auth_api_client
 ) -> None:
     (mock_api_client, project_test_data, _, _) = mock_up_tuple
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
+    session_controller = SessionController()
+    session_controller.api_client = mock_api_client
     session_controller._list_app_configs = Mock(  # type: ignore
         return_value=[
             AppConfig(
@@ -649,13 +654,12 @@ def test_get_app_config_id_from_name(
 @pytest.mark.parametrize("build_identifier", ["my_app_config:1", "my_app_config"])
 def test_get_build_id_from_build_identifier(
     mock_up_tuple: Tuple[Mock, Project, Cloud, Mock],
-    base_mock_anyscale_api_client: Mock,
+    mock_auth_api_client,
     build_identifier: str,
 ) -> None:
     (mock_api_client, project_test_data, _, _) = mock_up_tuple
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
+    session_controller = SessionController()
+    session_controller.api_client = mock_api_client
     session_controller._get_app_config_id_from_name = Mock(  # type: ignore
         return_value="mock_app_config_id"
     )
@@ -701,14 +705,13 @@ def test_get_build_id_from_build_identifier(
 @pytest.mark.parametrize("use_build_id", [False, True])
 def test_anyscale_start(
     mock_up_tuple: Tuple[Mock, Project, Cloud, Mock],
-    base_mock_anyscale_api_client: Mock,
+    mock_auth_api_client,
     sync_files: bool,
     use_build_id: bool,
 ) -> None:
     (mock_api_client, project_test_data, _, _,) = mock_up_tuple
-    session_controller = SessionController(
-        api_client=mock_api_client, anyscale_api_client=base_mock_anyscale_api_client
-    )
+    session_controller = SessionController()
+    session_controller.api_client = mock_api_client
     session_controller.push = Mock()  # type: ignore
     session_controller._get_or_generate_session_name = Mock(return_value="session-11")  # type: ignore
     session_controller._register_compute_config = Mock(  # type: ignore
@@ -765,7 +768,7 @@ def test_anyscale_start(
             "mock_compute_template_id",
             -1,
         )
-        base_mock_anyscale_api_client.start_session.assert_called_once_with(
+        session_controller.anyscale_api_client.start_session.assert_called_once_with(
             "session-11-id",
             StartSessionOptions(
                 build_id="mock_build_id", compute_template_id="mock_compute_template_id"
@@ -778,15 +781,19 @@ def test_anyscale_start(
             )
 
 
+@pytest.mark.parametrize("force_head_node", [True, False])
+@pytest.mark.parametrize("on_head_node", [True, False])
+@pytest.mark.parametrize("worker_node_id", [None, "123456"])
+@pytest.mark.parametrize("worker_node_ip", [None, "0.0.0.0"])
 def test_ssh_session(
     session_test_data: Session,
-    mock_api_client_multiple_sessions: Mock,
-    base_mock_anyscale_api_client: Mock,
+    mock_auth_api_client_multiple_sessions,
+    on_head_node: bool,
+    worker_node_id: Optional[str],
+    worker_node_ip: Optional[str],
+    force_head_node: bool,
 ) -> None:
-    session_controller = SessionController(
-        api_client=mock_api_client_multiple_sessions,
-        anyscale_api_client=base_mock_anyscale_api_client,
-    )
+    session_controller = SessionController()
 
     mock_project_definition = Mock()
     mock_project_definition.root = "/some/directory"
@@ -794,9 +801,19 @@ def test_ssh_session(
 
     mock_get_project_id = Mock(return_value=session_test_data.project_id)
     mock_get_project_session = Mock(return_value=session_test_data)
-    mock_get_cluster_config = Mock(
-        return_value={"auth": {"ssh_user": "", "ssh_private_key": ""}}
-    )
+    mock_cluster_config = {
+        "auth": {"ssh_user": "ubuntu", "ssh_private_key": ""},
+        "docker": {"container_name": "ray_container"},
+    }
+    mock_get_cluster_config = Mock(return_value=mock_cluster_config)
+    mock_ray_init = Mock()
+    mock_node_manager_address = "111.111.111.111"
+    mock_node_data = [
+        {"NodeID": "123456", "NodeManagerAddress": mock_node_manager_address}
+    ]
+    mock_ray_nodes = Mock(return_value=mock_node_data)
+
+    mock_exists = Mock(return_value=on_head_node)
 
     with patch.multiple(
         "anyscale.controllers.session_controller",
@@ -804,30 +821,63 @@ def test_ssh_session(
         get_project_id=mock_get_project_id,
         get_cluster_config=mock_get_cluster_config,
         get_project_session=mock_get_project_session,
-    ), patch.multiple("subprocess", run=Mock()):
-        session_controller.ssh(session_name=session_test_data.name, ssh_option=("",))
+    ), patch.multiple("subprocess", run=Mock()), patch(
+        "builtins.open", mock_open(read_data=str(mock_cluster_config))
+    ), patch.multiple(
+        "ray", init=mock_ray_init, nodes=mock_ray_nodes
+    ), patch.multiple(
+        "os.path", exists=mock_exists
+    ):
+        if force_head_node and not on_head_node:
+            # It's the user's responsibility to never do this
+            return
+        if (force_head_node and worker_node_id is None and worker_node_ip is None) or (
+            worker_node_id is not None and worker_node_ip is not None
+        ):
+            with pytest.raises(click.ClickException):
+                session_controller.ssh(
+                    session_name=session_test_data.name,
+                    ssh_option=("",),
+                    worker_node_id=worker_node_id,
+                    worker_node_ip=worker_node_ip,
+                    force_head_node=force_head_node,
+                )
+            return
+        else:
+            session_controller.ssh(
+                session_name=session_test_data.name,
+                ssh_option=("",),
+                worker_node_id=worker_node_id,
+                worker_node_ip=worker_node_ip,
+                force_head_node=force_head_node,
+            )
 
-    mock_api_client_multiple_sessions.get_session_head_ip_api_v2_sessions_session_id_head_ip_get.assert_called_once_with(
-        session_test_data.id,
-    )
-
-    mock_load_project_or_throw.assert_called_once_with()
-    mock_get_project_id.assert_called_once_with("/some/directory")
-    mock_get_cluster_config.assert_called_once_with(
-        session_test_data.name,
-        mock_api_client_multiple_sessions,
-        disable_project_sync=True,
-    )
+    if on_head_node and worker_node_id is not None:
+        mock_ray_init.assert_called_once_with(address="auto")
+        mock_ray_nodes.assert_called_once_with()
+    elif on_head_node and worker_node_ip is not None:
+        mock_ray_init.assert_not_called()
+        mock_ray_nodes.assert_not_called()
+    elif worker_node_id is not None or worker_node_ip is not None:
+        session_controller.api_client.get_session_head_ip_api_v2_sessions_session_id_head_ip_get.assert_called_once_with(
+            session_test_data.id,
+        )
+    else:
+        session_controller.api_client.get_session_head_ip_api_v2_sessions_session_id_head_ip_get.assert_called_once_with(
+            session_test_data.id,
+        )
+        mock_load_project_or_throw.assert_called_once_with()
+        mock_get_project_id.assert_called_once_with("/some/directory")
+        mock_get_cluster_config.assert_called_once_with(
+            session_test_data.name,
+            session_controller.api_client,
+            disable_project_sync=True,
+        )
 
 
 def test_autopush(
-    session_test_data: Session,
-    mock_api_client_with_session: Mock,
-    base_mock_anyscale_api_client: Mock,
+    session_test_data: Session, mock_auth_api_client_with_session
 ) -> None:
-    mock_api_client_with_session.get_session_head_ip_api_v2_sessions_session_id_head_ip_get.return_value.result.head_ip = (
-        "127.0.0.1"
-    )
 
     mock_project_definition = Mock()
     mock_project_definition.root = "/some/directory"
@@ -847,9 +897,9 @@ def test_autopush(
     mock_perform_sync = Mock()
     mock_perform_sync.return_value.returncode = 0
 
-    session_controller = SessionController(
-        api_client=mock_api_client_with_session,
-        anyscale_api_client=base_mock_anyscale_api_client,
+    session_controller = SessionController()
+    session_controller.api_client.get_session_head_ip_api_v2_sessions_session_id_head_ip_get.return_value.result.head_ip = (
+        "127.0.0.1"
     )
 
     with patch.multiple(
@@ -866,7 +916,7 @@ def test_autopush(
     ):
         session_controller.autopush(session_name=session_test_data.name)
 
-    mock_api_client_with_session.get_session_head_ip_api_v2_sessions_session_id_head_ip_get.assert_called_once_with(
+    session_controller.api_client.get_session_head_ip_api_v2_sessions_session_id_head_ip_get.assert_called_once_with(
         session_test_data.id,
     )
 
@@ -875,21 +925,21 @@ def test_autopush(
     mock_get_project_session.assert_called_once_with(
         session_test_data.project_id,
         session_test_data.name,
-        mock_api_client_with_session,
+        session_controller.api_client,
     )
     mock_get_cluster_config.assert_called_once_with(
-        session_test_data.name, mock_api_client_with_session
+        session_test_data.name, session_controller.api_client
     )
     mock_wait_for_session_start.assert_called_once_with(
         session_test_data.project_id,
         session_test_data.name,
-        mock_api_client_with_session,
+        session_controller.api_client,
     )
     mock_get_working_dir.assert_called_once_with(
-        cluster_config_data, session_test_data.project_id, mock_api_client_with_session
+        cluster_config_data, session_test_data.project_id, session_controller.api_client
     )
     mock_managed_autosync_session.assert_called_once_with(
-        session_test_data.id, mock_api_client_with_session
+        session_test_data.id, session_controller.api_client
     )
     mock_perform_sync.assert_called_once_with(
         [
@@ -915,13 +965,8 @@ def test_autopush(
 
 
 def test_autosync_docker(
-    session_test_data: Session,
-    mock_api_client_with_session: Mock,
-    base_mock_anyscale_api_client: Mock,
+    session_test_data: Session, mock_auth_api_client_with_session
 ) -> None:
-    mock_api_client_with_session.get_session_head_ip_api_v2_sessions_session_id_head_ip_get.return_value.result.head_ip = (
-        "127.0.0.1"
-    )
 
     mock_project_definition = Mock()
     mock_project_definition.root = "/some/directory"
@@ -945,9 +990,9 @@ def test_autosync_docker(
     mock_perform_sync = Mock()
     mock_perform_sync.return_value.returncode = 0
 
-    session_controller = SessionController(
-        api_client=mock_api_client_with_session,
-        anyscale_api_client=base_mock_anyscale_api_client,
+    session_controller = SessionController()
+    session_controller.api_client.get_session_head_ip_api_v2_sessions_session_id_head_ip_get.return_value.result.head_ip = (
+        "127.0.0.1"
     )
 
     remote_location = "/fake/host/mount"
@@ -970,7 +1015,7 @@ def test_autosync_docker(
     ), patch.multiple("subprocess", run=mock_subprocess_run):
         session_controller.autopush(session_name=session_test_data.name)
 
-    mock_api_client_with_session.get_session_head_ip_api_v2_sessions_session_id_head_ip_get.assert_called_once_with(
+    session_controller.api_client.get_session_head_ip_api_v2_sessions_session_id_head_ip_get.assert_called_once_with(
         session_test_data.id,
     )
 
@@ -979,21 +1024,21 @@ def test_autosync_docker(
     mock_get_project_session.assert_called_once_with(
         session_test_data.project_id,
         session_test_data.name,
-        mock_api_client_with_session,
+        session_controller.api_client,
     )
     mock_get_cluster_config.assert_called_once_with(
-        session_test_data.name, mock_api_client_with_session
+        session_test_data.name, session_controller.api_client
     )
     mock_wait_for_session_start.assert_called_once_with(
         session_test_data.project_id,
         session_test_data.name,
-        mock_api_client_with_session,
+        session_controller.api_client,
     )
     mock_get_working_dir.assert_called_once_with(
-        cluster_config_data, session_test_data.project_id, mock_api_client_with_session
+        cluster_config_data, session_test_data.project_id, session_controller.api_client
     )
     mock_managed_autosync_session.assert_called_once_with(
-        session_test_data.id, mock_api_client_with_session
+        session_test_data.id, session_controller.api_client
     )
     mock_perform_sync.assert_called_once_with(
         [
@@ -1019,9 +1064,7 @@ def test_autosync_docker(
 
 
 def test_anyscale_push_session(
-    session_test_data: Session,
-    mock_api_client_with_session: Mock,
-    base_mock_anyscale_api_client: Mock,
+    session_test_data: Session, mock_auth_api_client_with_session
 ) -> None:
     mock_project_definition = Mock()
     mock_project_definition.root = "/some/directory"
@@ -1042,10 +1085,7 @@ def test_anyscale_push_session(
     mock_rsync = Mock()
     mock_up = Mock()
 
-    session_controller = SessionController(
-        api_client=mock_api_client_with_session,
-        anyscale_api_client=base_mock_anyscale_api_client,
-    )
+    session_controller = SessionController()
 
     with patch.multiple("subprocess", run=mock_subprocess_run), patch.multiple(
         "anyscale.controllers.session_controller",
@@ -1070,7 +1110,7 @@ def test_anyscale_push_session(
         )
 
         mock_validate_cluster_configuration.assert_called_once_with(
-            config, api_instance=mock_api_client_with_session
+            config, api_instance=session_controller.api_client
         )
 
         mock_up.assert_called_once_with(
@@ -1091,10 +1131,7 @@ def test_anyscale_push_session(
 
 @pytest.mark.parametrize("config", [None, "tmp.yaml"])
 def test_pull(
-    session_test_data: Session,
-    mock_api_client_with_session: Mock,
-    base_mock_anyscale_api_client: Mock,
-    config: str,
+    session_test_data: Session, mock_auth_api_client_with_session, config: str,
 ) -> None:
     mock_project_definition = Mock()
     mock_project_definition.root = "/some/directory"
@@ -1105,13 +1142,10 @@ def test_pull(
     mock_get_working_dir = Mock(return_value="directory")
     mock_rsync = Mock()
     mock_get_project_session = Mock(return_value=session_test_data)
-    mock_api_client_with_session.get_session_cluster_config_api_v2_sessions_session_id_cluster_config_get.return_value.result.config_with_defaults = (
-        "temp"
-    )
 
-    session_controller = SessionController(
-        api_client=mock_api_client_with_session,
-        anyscale_api_client=base_mock_anyscale_api_client,
+    session_controller = SessionController()
+    session_controller.api_client.get_session_cluster_config_api_v2_sessions_session_id_cluster_config_get.return_value.result.config_with_defaults = (
+        "temp"
     )
 
     with patch.multiple(
@@ -1128,59 +1162,28 @@ def test_pull(
     mock_load_project_or_throw.assert_called_once_with()
     mock_get_project_id.assert_called_once_with("/some/directory")
     mock_get_cluster_config.assert_called_once_with(
-        session_test_data.name, mock_api_client_with_session
+        session_test_data.name, session_controller.api_client
     )
     mock_get_working_dir.assert_called_once_with(
-        cluster_config_data, session_test_data.project_id, mock_api_client_with_session
+        cluster_config_data, session_test_data.project_id, session_controller.api_client
     )
     mock_rsync.assert_called_once()
     if config:
         mock_get_project_session.assert_called_once_with(
             session_test_data.project_id,
             session_test_data.name,
-            mock_api_client_with_session,
+            session_controller.api_client,
         )
-        mock_api_client_with_session.get_session_cluster_config_api_v2_sessions_session_id_cluster_config_get.assert_called_once_with(
+        session_controller.api_client.get_session_cluster_config_api_v2_sessions_session_id_cluster_config_get.assert_called_once_with(
             session_test_data.id
         )
 
 
-@pytest.mark.parametrize("project_name", ["project_name", None])
-def test_fork_session(
-    session_test_data: Session,
-    mock_api_client_with_session: Mock,
-    base_mock_anyscale_api_client: Mock,
-    project_name: Optional[str],
-) -> None:
-
-    with patch.object(
-        SessionController, "_resolve_session", return_value=session_test_data
-    ) as mock_resolve_session, patch.object(
-        SessionController, "_fork_session_internal", return_value=session_test_data
-    ) as mock_fork_session_internal:
-        session_controller = SessionController(
-            api_client=mock_api_client_with_session,
-            anyscale_api_client=base_mock_anyscale_api_client,
-        )
-        output_str = session_controller.fork_session(
-            "session_name", "new_session_name", project_name
-        )
-
-    mock_resolve_session.assert_called_once_with("session_name", project_name)
-    mock_fork_session_internal.assert_called_once_with(
-        session_test_data, "new_session_name"
-    )
-
-    url = f"https://beta.anyscale.com/projects/{session_test_data.project_id}/sessions/{session_test_data.id}"
-    assert output_str == f"Session {session_test_data.name} started. View at {url}"
-
-
 @pytest.mark.parametrize("project_name", [None, "project_name"])
 def test_resolve_session(
     project_test_data: Project,
     session_test_data: Session,
-    mock_api_client_with_session: Mock,
-    base_mock_anyscale_api_client: Mock,
+    mock_auth_api_client_with_session,
     project_name: Optional[str],
 ) -> None:
     mock_project_definition = Mock()
@@ -1189,10 +1192,11 @@ def test_resolve_session(
 
     mock_get_project_id = Mock(return_value=session_test_data.project_id)
 
-    mock_api_client_with_session.list_sessions_api_v2_sessions_get.return_value = SessionListResponse(
+    session_controller = SessionController()
+    session_controller.api_client.list_sessions_api_v2_sessions_get.return_value = SessionListResponse(
         results=[session_test_data]
     )
-    mock_api_client_with_session.list_projects_api_v2_projects_get.return_value = ProjectListResponse(
+    session_controller.api_client.list_projects_api_v2_projects_get.return_value = ProjectListResponse(
         results=[project_test_data]
     )
 
@@ -1201,10 +1205,6 @@ def test_resolve_session(
         load_project_or_throw=mock_load_project_or_throw,
         get_project_id=mock_get_project_id,
     ):
-        session_controller = SessionController(
-            api_client=mock_api_client_with_session,
-            anyscale_api_client=base_mock_anyscale_api_client,
-        )
         session = session_controller._resolve_session("session_name", project_name)
 
     assert session == session_test_data
@@ -1212,46 +1212,12 @@ def test_resolve_session(
     if project_name:
         mock_load_project_or_throw.assert_not_called()
         mock_get_project_id.assert_not_called()
-        mock_api_client_with_session.list_projects_api_v2_projects_get.assert_called_once_with()
+        session_controller.api_client.list_projects_api_v2_projects_get.assert_called_once_with()
     else:
         mock_load_project_or_throw.assert_called_once()
         mock_get_project_id.assert_called_once_with(mock_project_definition.root)
-        mock_api_client_with_session.list_projects_api_v2_projects_get.assert_not_called()
+        session_controller.api_client.list_projects_api_v2_projects_get.assert_not_called()
 
-    mock_api_client_with_session.list_sessions_api_v2_sessions_get.assert_called_once_with(
+    session_controller.api_client.list_sessions_api_v2_sessions_get.assert_called_once_with(
         session_test_data.project_id, name="session_name"
     )
-
-
-def test_fork_session_internal(
-    session_test_data: Session,
-    mock_api_client_with_session: Mock,
-    base_mock_anyscale_api_client: Mock,
-) -> None:
-    mock_api_client_with_session.fork_session_api_v2_sessions_session_id_fork_post.return_value = (
-        None
-    )
-    mock_api_client_with_session.list_sessions_api_v2_sessions_get.return_value = SessionListResponse(
-        results=[session_test_data]
-    )
-
-    session_controller = SessionController(
-        api_client=mock_api_client_with_session,
-        anyscale_api_client=base_mock_anyscale_api_client,
-    )
-    cloned_session = session_controller._fork_session_internal(
-        session_test_data, "new_session_name"
-    )
-
-    assert cloned_session == session_test_data
-
-    mock_api_client_with_session.fork_session_api_v2_sessions_session_id_fork_post.assert_called_once_with(
-        session_id=session_test_data.id,
-        create_session_from_snapshot_options=CreateSessionFromSnapshotOptions(
-            project_id=session_test_data.project_id, name="new_session_name"
-        ),
-        _request_timeout=300000,
-    )
-    mock_api_client_with_session.list_sessions_api_v2_sessions_get.assert_called_once_with(
-        project_id=session_test_data.project_id, name="new_session_name"
-    )
