@@ -4,6 +4,7 @@ from unittest.mock import call, Mock, patch
 
 import pytest
 
+from anyscale.authenticate import get_auth_api_client
 from anyscale.autosync_heartbeat import (
     managed_autosync_session,
     perform_autopush_synchronization,
@@ -11,12 +12,19 @@ from anyscale.autosync_heartbeat import (
 
 
 def test_managed_autosync_session() -> None:
-    mock_api_client = Mock()
-    mock_api_client.register_autosync_session_api_v2_autosync_sessions_post.return_value.result.id = (
-        1
+    with patch.multiple(
+        "anyscale.authenticate.AuthenticationBlock",
+        _validate_api_client_auth=Mock(),
+        _validate_credentials_format=Mock(),
+    ):
+        mock_api_client = get_auth_api_client(
+            cli_token="fake_cli_token", host="fake_host"
+        ).api_client
+    mock_api_client.register_autosync_session_api_v2_autosync_sessions_post = Mock(
+        return_value=Mock(result=Mock(id=1))
     )
-    mock_api_client.deregister_autosync_session_api_v2_autosync_sessions_autosync_session_id_delete.return_value = (
-        None
+    mock_api_client.deregister_autosync_session_api_v2_autosync_sessions_autosync_session_id_delete = Mock(
+        return_value=None
     )
 
     with managed_autosync_session("id", mock_api_client):
