@@ -104,7 +104,7 @@ def test_wait_for_session_start_error(
             mock_api_client_with_session,
         )
 
-    assert "Error while starting session" in e.value.message
+    assert "Error while starting cluster" in e.value.message
     mock_api_client_with_session.list_sessions_api_v2_sessions_get.assert_called_once_with(
         project_id=session_start_error_test_data.project_id,
         name=session_start_error_test_data.id,
@@ -125,7 +125,7 @@ def test_wait_for_session_start_terminated(
             mock_api_client_with_session,
         )
 
-    assert "Session is still in stopped/terminated state" in e.value.message
+    assert "Cluster is still in stopped/terminated state" in e.value.message
     mock_api_client_with_session.list_sessions_api_v2_sessions_get.assert_called_with(
         project_id=session_terminated_test_data.project_id,
         name=session_terminated_test_data.id,
@@ -146,7 +146,7 @@ def test_wait_for_session_start_missing_session(
             mock_api_client_with_session,
         )
 
-    assert "Session doesn't exist" in e.value.message
+    assert "Cluster doesn't exist" in e.value.message
     mock_api_client_with_session.list_sessions_api_v2_sessions_get.assert_called_once_with(
         project_id=session_start_error_test_data.project_id,
         name=session_start_error_test_data.id,
